@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2018 Richard Hughes <richard@hughsie.com>
+ * Copyright 2018 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "config.h"
@@ -175,7 +175,7 @@ xb_common_content_type_func(void)
 		     {"test.xml", "application/xml"},
 		     {"test.xml.gz.gz.gz", "application/gzip"},
 		     {"test.xml.xz", "application/x-xz"},
-		     {"test.xml.zstd", "application/zstd"},
+		     {"test.xml.zst", "application/zstd"},
 		     {NULL, NULL}};
 	for (guint i = 0; items[i].fn != NULL; i++) {
 		gboolean ret;
@@ -497,6 +497,12 @@ xb_builder_source_lzma_func(void)
 	g_autoptr(XbBuilderSource) source = xb_builder_source_new();
 	g_autoptr(XbSilo) silo = NULL;
 
+#ifndef HAVE_LZMA
+	/* not supported */
+	g_test_skip("compiled without -Dlzma");
+	return;
+#endif
+
 	/* import a source file */
 	path = g_test_build_filename(G_TEST_DIST, "test.xml.xz", NULL);
 	file_src = g_file_new_for_path(path);
@@ -532,7 +538,7 @@ xb_builder_source_zstd_func(void)
 	g_autoptr(XbSilo) silo = NULL;
 
 	/* import a source file */
-	path = g_test_build_filename(G_TEST_DIST, "test.xml.zstd", NULL);
+	path = g_test_build_filename(G_TEST_DIST, "test.xml.zst", NULL);
 	file_src = g_file_new_for_path(path);
 	if (!g_file_query_exists(file_src, NULL)) {
 		g_test_skip("does not work in subproject test");
@@ -661,6 +667,9 @@ xb_builder_ensure_func(void)
 {
 	gboolean ret;
 	guint invalidate_cnt = 0;
+	g_autofree gchar *bytes1str = NULL;
+	g_autofree gchar *bytes2str = NULL;
+	g_autofree gchar *bytes3str = NULL;
 	g_autofree gchar *tmp_xmlb = g_build_filename(g_get_tmp_dir(), "temp.xmlb", NULL);
 	g_autoptr(GBytes) bytes1 = NULL;
 	g_autoptr(GBytes) bytes2 = NULL;
@@ -713,6 +722,7 @@ xb_builder_ensure_func(void)
 			 &invalidate_cnt);
 	g_assert_cmpint(invalidate_cnt, ==, 0);
 	bytes1 = xb_silo_get_bytes(silo);
+	bytes1str = g_compute_checksum_for_bytes(G_CHECKSUM_SHA1, bytes1);
 
 	/* recreate file if it is invalid */
 	ret = g_file_replace_contents(file,
@@ -736,7 +746,8 @@ xb_builder_ensure_func(void)
 	g_assert_nonnull(silo);
 	g_assert_true(xb_silo_is_valid(silo));
 	bytes2 = xb_silo_get_bytes(silo);
-	g_assert(bytes1 != bytes2);
+	bytes2str = g_compute_checksum_for_bytes(G_CHECKSUM_SHA1, bytes2);
+	g_assert_cmpstr(bytes1str, !=, bytes2str);
 	g_clear_object(&silo);
 
 	/* don't recreate file if perfectly valid */
@@ -745,7 +756,8 @@ xb_builder_ensure_func(void)
 	g_assert_nonnull(silo);
 	g_assert_true(xb_silo_is_valid(silo));
 	bytes3 = xb_silo_get_bytes(silo);
-	g_assert(bytes2 == bytes3);
+	bytes3str = g_compute_checksum_for_bytes(G_CHECKSUM_SHA1, bytes3);
+	g_assert_cmpstr(bytes2str, ==, bytes3str);
 	g_clear_object(&silo);
 	g_clear_object(&builder);
 
